#include "render.h" 
EGLDisplay glesDisplay;  
EGLSurface glesSurface;	 
EGLContext glesContext;	 

//Variables declared in main.cpp, but used here too
extern HWND hWnd;
extern HDC hDC;
extern bool drawInOrtho;
extern bool drawTriangle;
static board* GameBoard;
static GLfloat capVert[80];
static GLfloat outerVert[120];
static GLfloat innerVert[120];
static GLfloat blockVertb[18];
static GLfloat blockVertl[12];
static GLfloat blockVertf[21];
static GLfloat blockVertr[12];
static GLfloat blockVertt[18];
static GLfloat blockVertba[21];
static bool texLoaded = false;
static bool capInit,blockInit,outInit,inInit = false;
static GLfloat	rtri;
static GLuint texture[1];
static GLfloat rtfi = 0.0f;
float bgVertices[] = {
	100.0f, -100.0f, 
	100.0f,  100.0f, 
	-100.0f, -100.0f, 
	-100.0f,  100.0f
};

float bgTexCoords[] = {
	3.0f, 0.0f,
	3.0f, 3.0f,
	0.0f, 0.0f,
	0.0f, 3.0f
};

unsigned char *loadBMP(char *filename, BITMAPINFOHEADER *bmpInfo)
{
	FILE *file;
	BITMAPFILEHEADER bmpFile;
	unsigned char *bmpImage = NULL;
	unsigned char tmpRGB;

	TCHAR path[256];
	char fullPath[256];
	GetModuleFileName(NULL, path, 256);

	TCHAR *pos = wcsrchr(path, '\\');
	*(pos + 1) = '\0';

	wcstombs(fullPath, path, 256);

	strcat(fullPath, filename);

	file = fopen(fullPath,"rb");
	if (!file)
	{
		MessageBox(NULL, L"Can't Find Bitmap", L"Error", MB_OK);
		return NULL;
	}

	fread(&bmpFile,sizeof(BITMAPFILEHEADER),1,file);

	if (bmpFile.bfType != 0x4D42)
	{
		MessageBox(NULL, L"Incorrect texture type", L"Error", MB_OK);
		fclose(file);
		return NULL;
	}

	fread(bmpInfo,sizeof(BITMAPINFOHEADER),1,file);

	fseek(file,bmpFile.bfOffBits,SEEK_SET);

	bmpImage = new unsigned char[bmpInfo->biSizeImage];

	if (!bmpImage)
	{
		MessageBox(NULL, L"Out of Memory", L"Error", MB_OK);
		delete[] bmpImage;
		fclose(file);
		return NULL;
	}

	fread(bmpImage,1,bmpInfo->biSizeImage,file);

	if (!bmpImage)
	{
		MessageBox(NULL, L"Error reading bitmap", L"Error", MB_OK);
		fclose(file);
		return NULL;
	}

	for (unsigned int i = 0; i < bmpInfo->biSizeImage; i+=3)
	{
		tmpRGB = bmpImage[i];
		bmpImage[i] = bmpImage[i+2];
		bmpImage[i+2] = tmpRGB;
	}

	fclose(file);

	return bmpImage;
}

bool loadTextures()
{
	if(texLoaded)
	{
		return true;
	}
	BITMAPINFOHEADER info;
	unsigned char *bitmap = NULL;
	glGenTextures(1, texture);


	memset(&info, 0, sizeof(BITMAPINFOHEADER));

	bitmap = loadBMP("bg.BMP", &info);

	if (!bitmap)
		return false;

	glBindTexture(GL_TEXTURE_2D, texture[0]);

	glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, info.biWidth,
		info.biHeight, 0, GL_RGB, GL_UNSIGNED_BYTE,
		bitmap);

	glTexParameterf(GL_TEXTURE_2D, 
		GL_TEXTURE_MIN_FILTER, GL_LINEAR);
	glTexParameterf(GL_TEXTURE_2D,
		GL_TEXTURE_MAG_FILTER, GL_LINEAR);

	delete[] bitmap;
	bitmap = NULL;


	return true;
}
bool InitDraw(board* a)
{  
	EGLConfig configs[10];
	EGLint matchingConfigs;	
	GameBoard = a;
	const EGLint configAttribs[] =
	{
		EGL_RED_SIZE,       8,
		EGL_GREEN_SIZE,     8,
		EGL_BLUE_SIZE,      8,
		EGL_ALPHA_SIZE,     EGL_DONT_CARE,
		EGL_DEPTH_SIZE,     16,
		EGL_STENCIL_SIZE,   EGL_DONT_CARE,
		EGL_SURFACE_TYPE,   EGL_WINDOW_BIT,
		EGL_NONE,           EGL_NONE
	};



	hDC = GetWindowDC(hWnd);

	glesDisplay = eglGetDisplay(hDC);

	if(!eglInitialize(glesDisplay, NULL, NULL))
	{
		return false;
	}

	if(!eglChooseConfig(glesDisplay, configAttribs, &configs[0], 10, &matchingConfigs)) 
	{
		return false;
	}

	if (matchingConfigs < 1)  return false;

	glesSurface = eglCreateWindowSurface(glesDisplay, configs[0], hWnd, configAttribs);

	if(!glesSurface) return false; 

	glesContext = eglCreateContext(glesDisplay, configs[0], 0, configAttribs);

	if(!glesContext) return false;

	eglMakeCurrent(glesDisplay, glesSurface, glesSurface, glesContext);
	if(!texLoaded)
	{
		if (!loadTextures())
		{
			MessageBox(NULL, L"Error loading textures", L"Error", MB_OK);
			return false;
		}
		texLoaded=true;
	}

	glClearColorx(0, 0, 0, 0);

	//Do not want to see smoothed colors, only a solid color for face
	glShadeModel(GL_SMOOTH);

	//Enable the depth test in order to see the cube correctly
	//glEnable(GL_DEPTH_TEST);

	/*Taking care of specifying correctly the winding order of the 
	vertices (counter clockwise order), we can cull up all back faces.
	This is probably one of the better optimizations we could work,
	because we are avoiding a lot of computations that wont be reflected
	in the screen, using  glEnable(GL_CULL_FACE) to do the work*/
	//glEnable(GL_CULL_FACE);
	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	RECT r;
	GetWindowRect(hWnd, &r);
	glViewport(r.left, r.top, r.right - r.left, r.bottom - r.top);

	SetOrtho();
	return true;
}
void initBlock(GLfloat inner, GLfloat outer,GLfloat height, GLint slices)
{
	int x = 0;
	GLfloat angle;
	GLfloat sinCache[3];
	GLfloat cosCache[3];
	sinCache[0] = (GLfloat)sin(0);
	cosCache[0] = (GLfloat)cos(0);
	angle = (2 * PI * (slices-1)/slices);
	sinCache[1] = (GLfloat)(sin(2 * PI * 1 / slices)+sin(0))/2;
	cosCache[1] = (GLfloat)(cos(2 * PI * 1 / slices)+cos(0))/2;
	sinCache[2] = (GLfloat)(sin(2 * PI * (slices - 1) / slices)+sin(0))/2;
	cosCache[2] = (GLfloat)(cos(2 * PI * (slices - 1) / slices)+cos(0))/2;
	if(! blockInit)
	{
		blockInit = true;
		blockVertb[x++] = outer*sinCache[2];
		blockVertb[x++] = outer*cosCache[2];
		blockVertb[x++] = 0;
		blockVertb[x++] = inner*sinCache[2];
		blockVertb[x++] = inner*cosCache[2];
		blockVertb[x++] = 0;
		blockVertb[x++] = outer*sinCache[0];
		blockVertb[x++] = outer*cosCache[0];
		blockVertb[x++] = 0;
		blockVertb[x++] = inner*sinCache[0];
		blockVertb[x++] = inner*cosCache[0];
		blockVertb[x++] = 0;
		blockVertb[x++] = outer*sinCache[1];
		blockVertb[x++] = outer*cosCache[1];
		blockVertb[x++] = 0;
		blockVertb[x++] = inner*sinCache[1];
		blockVertb[x++] = inner*cosCache[1];
		blockVertb[x++] = 0;
		x = 0;
		blockVertl[x++] = outer*sinCache[2];
		blockVertl[x++] = outer*cosCache[2];
		blockVertl[x++] = 0;
		blockVertl[x++] = inner*sinCache[2];
		blockVertl[x++] = inner*cosCache[2];
		blockVertl[x++] = 0;
		blockVertl[x++] = outer*sinCache[2];
		blockVertl[x++] = outer*cosCache[2];
		blockVertl[x++] = height;
		blockVertl[x++] = inner*sinCache[2];
		blockVertl[x++] = inner*cosCache[2];
		blockVertl[x++] = height;
		x=0;
		blockVertf[x++] = outer*sinCache[2];
		blockVertf[x++] = outer*cosCache[2];
		blockVertf[x++] = 0;
		blockVertf[x++] = outer*sinCache[2];
		blockVertf[x++] = outer*cosCache[2];
		blockVertf[x++] = height;
		blockVertf[x++] = outer*sinCache[0];
		blockVertf[x++] = outer*cosCache[0];
		blockVertf[x++] = 0;
		blockVertf[x++] = outer*sinCache[0];
		blockVertf[x++] = outer*cosCache[0];
		blockVertf[x++] = height;
		blockVertf[x++] = outer*sinCache[1];
		blockVertf[x++] = outer*cosCache[1];
		blockVertf[x++] = 0;
		blockVertf[x++] = outer*sinCache[1];
		blockVertf[x++] = outer*cosCache[1];
		blockVertf[x++] = height;
		blockVertf[x++] = outer*sinCache[0];
		blockVertf[x++] = outer*cosCache[0];
		blockVertf[x++] = 0;
		x=0;
		blockVertr[x++] = outer*sinCache[1];
		blockVertr[x++] = outer*cosCache[1];
		blockVertr[x++] = 0;
		blockVertr[x++] = inner*sinCache[1];
		blockVertr[x++] = inner*cosCache[1];
		blockVertr[x++] = 0;
		blockVertr[x++] = outer*sinCache[1];
		blockVertr[x++] = outer*cosCache[1];
		blockVertr[x++] = height;
		blockVertr[x++] = inner*sinCache[1];
		blockVertr[x++] = inner*cosCache[1];
		blockVertr[x++] = height;
		x=0;
		blockVertt[x++] = outer*sinCache[2];
		blockVertt[x++] = outer*cosCache[2];
		blockVertt[x++] = height;
		blockVertt[x++] = inner*sinCache[2];
		blockVertt[x++] = inner*cosCache[2];
		blockVertt[x++] = height;
		blockVertt[x++] = outer*sinCache[0];
		blockVertt[x++] = outer*cosCache[0];
		blockVertt[x++] = height;
		blockVertt[x++] = inner*sinCache[0];
		blockVertt[x++] = inner*cosCache[0];
		blockVertt[x++] = height;
		blockVertt[x++] = outer*sinCache[1];
		blockVertt[x++] = outer*cosCache[1];
		blockVertt[x++] = height;
		blockVertt[x++] = inner*sinCache[1];
		blockVertt[x++] = inner*cosCache[1];
		blockVertt[x++] = height;
		x=0;
		blockVertba[x++] = inner*sinCache[2];
		blockVertba[x++] = inner*cosCache[2];
		blockVertba[x++] = 0;
		blockVertba[x++] = inner*sinCache[2];
		blockVertba[x++] = inner*cosCache[2];
		blockVertba[x++] = height;
		blockVertba[x++] = inner*sinCache[0];
		blockVertba[x++] = inner*cosCache[0];
		blockVertba[x++] = 0;
		blockVertba[x++] = inner*sinCache[0];
		blockVertba[x++] = inner*cosCache[0];
		blockVertba[x++] = height;
		blockVertba[x++] = inner*sinCache[1];
		blockVertba[x++] = inner*cosCache[1];
		blockVertba[x++] = 0;
		blockVertba[x++] = inner*sinCache[1];
		blockVertba[x++] = inner*cosCache[1];
		blockVertba[x++] = height;
		blockVertba[x++] = inner*sinCache[0];
		blockVertba[x++] = inner*cosCache[0];
		blockVertba[x++] = 0;
	}
	GLubyte topLines[] = {0,1,3,5,4,2,0};
	GLubyte baLines[] = {0,1,3,5,4,2,0};
	glEnableClientState(GL_VERTEX_ARRAY);
	glVertexPointer(3, GL_FLOAT, 0, blockVertb);
	glDrawArrays(GL_TRIANGLE_STRIP, 0, 6);
	glVertexPointer(3, GL_FLOAT, 0, blockVertt);
	glDrawArrays(GL_TRIANGLE_STRIP, 0, 6);
	glVertexPointer(3, GL_FLOAT, 0, blockVertf);
	glDrawArrays(GL_TRIANGLE_STRIP, 0, 7);
	glVertexPointer(3, GL_FLOAT, 0, blockVertba);
	glDrawArrays(GL_TRIANGLE_STRIP, 0, 7);
	glVertexPointer(3, GL_FLOAT, 0, blockVertr);
	glDrawArrays(GL_TRIANGLE_STRIP, 0, 4);
	glVertexPointer(3, GL_FLOAT, 0, blockVertl);
	glDrawArrays(GL_TRIANGLE_STRIP, 0, 4);
	glDisableClientState(GL_VERTEX_ARRAY);  
}
void drawBoard()
{
	int xsize = GameBoard->get_x();
	int ysize = GameBoard->get_y();
	sphere*** content = GameBoard->getboard();

	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);  
	glLoadIdentity();
	if(!drawTriangle)
	{
		glDisable(GL_BLEND);
		glEnable(GL_TEXTURE_2D);
		glColor4x(FixedFromFloat(0.8f),FixedFromFloat(0.8f),FixedFromFloat(0.8f),ONE);
		glVertexPointer(2, GL_FLOAT, 0, bgVertices);
		glTexCoordPointer(2, GL_FLOAT, 0, bgTexCoords);
		glEnableClientState(GL_VERTEX_ARRAY);
		glEnableClientState(GL_TEXTURE_COORD_ARRAY);
		glDrawArrays(GL_TRIANGLE_STRIP, 0, 4);
		glDisableClientState(GL_VERTEX_ARRAY);
		glDisableClientState(GL_TEXTURE_COORD_ARRAY);
		glDisable(GL_TEXTURE_2D);
		glEnable(GL_BLEND);
	}
	glTranslatef(-1.5f,20.0f,-6.0f);
	glRotatef(110,1,0,0);
	glColor4x(ONE,ONE,ONE,FixedFromFloat(0.3f));
	Cylinder(15,15,60,xsize,ysize);
	Cylinder(25,25,60,xsize,ysize);
	cap(15,25,xsize);
	int center = (GameBoard->getCobliPosition())-2;
	GLfloat yoff = (GLfloat) 60 / ysize;
	GLfloat xoff = (GLfloat) 360 / xsize;
	glEnable(GL_DEPTH_TEST);
	for(int i = (ysize - 1);i >= 0;i--)
	{
		for(int j = 0;j < xsize;j++)
		{
			glLoadIdentity();
			glTranslatef(-1.5f,20.0f,-6.0f);
			glRotatef(110,1,0,0);
			glTranslatef(0.0f,0.0f,i * yoff);
			glRotatef((xoff * (j-center))+180,0.0f,0.0f,1.0f);
			if(! (content[i][j]->color == NONE_))
			{	
				switch(content[i][j]->color)
				{
				case RED_:
					glColor4x(ONE,0,0,ONE);
					break;
				case BLUE_:
					glColor4x(0,0,ONE,ONE);
					break;
				case GREEN_:
					glColor4x(0,ONE,0,ONE);
					break;
				case YELLOW_:
					glColor4x(0,ONE,ONE,ONE);
					break;
				case ORANGE_:
					glColor4x(ONE,FixedFromFloat(0.64f),0,ONE);
					break;
				default:
					glColor4x(ONE,ONE,ONE,ONE);
					break;
				}
				initBlock(15,25,yoff,xsize);
			}
		}
	}
	glEnable(GL_CULL_FACE);
	glColor4x(ONE,ONE,ONE,FixedFromFloat(0.5f));
	Cylinder(15,15,60,xsize,ysize);
	glDisable(GL_CULL_FACE);
	glDisable(GL_DEPTH_TEST);
}
void cap(GLfloat innerRadius, GLfloat outerRadius, GLint slices)
{

	GLint i;
	GLfloat sinCache[240];
	GLfloat cosCache[240];
	GLfloat angle;
	int x = 0;
	if(! capInit)
	{
		capInit = true;
		if (slices >= 240) slices = 240-1;

		for (i = 0; i < slices; i++) {
			angle = 2 * PI * i / slices;
			sinCache[i] = (GLfloat)sin(angle);
			cosCache[i] = (GLfloat)cos(angle);
		}
		sinCache[slices] = sinCache[0];
		cosCache[slices] = cosCache[0];
		for (i = 0; i <= slices; i++) {

			capVert[x++]=innerRadius * sinCache[i];
			capVert[x++]=innerRadius * cosCache[i];
			capVert[x++]=0;
			capVert[x++]=outerRadius * sinCache[i];
			capVert[x++]=outerRadius * cosCache[i];
			capVert[x++]=0;
		}
	}
	glVertexPointer(3, GL_FLOAT, 0, capVert);
	glEnableClientState(GL_VERTEX_ARRAY);
	glDrawArrays(GL_TRIANGLE_STRIP, 0, (slices+1)*2);
	glDisableClientState(GL_VERTEX_ARRAY);
}
void Cylinder(GLfloat baseRadius, GLfloat topRadius, GLfloat height, GLint slices, GLint stacks)
{
	static int rotation = 0;
	GLint i,j;
	GLfloat sinCache[240];
	GLfloat cosCache[240];
	GLfloat sinCache3[240];
	GLfloat cosCache3[240];
	GLfloat vertices[240];
	GLfloat angle;
	GLfloat zLow, zHigh;
	GLfloat length;
	GLfloat deltaRadius;
	GLfloat zNormal;
	GLfloat xyNormalRatio;
	GLfloat radiusLow, radiusHigh;
	int x = 0;

	if (slices >= 240) slices = 240-1;

	deltaRadius = baseRadius - topRadius;
	length = (GLfloat)sqrt(deltaRadius*deltaRadius + height*height);


	zNormal = deltaRadius / length;
	xyNormalRatio = height / length;

	for (i = 0; i < slices; i++) {
		angle = 2 * PI * i / slices;
		sinCache[i] = (GLfloat)sin(angle);
		cosCache[i] = (GLfloat)cos(angle);
	}
	sinCache[slices] = sinCache[0];
	cosCache[slices] = cosCache[0];
	for (i = 0; i < slices; i++) {
		angle = 2 * PI * (i-0.5f) / slices;
		sinCache3[i] = xyNormalRatio * (GLfloat)sin(angle);
		cosCache3[i] = xyNormalRatio * (GLfloat)cos(angle);
	}
	sinCache3[slices] = sinCache3[0];
	cosCache3[slices] = cosCache3[0];
	for (j = 0; j < stacks; j++) {
		zLow = j * height / stacks;
		zHigh = (j + 1) * height / stacks;
		radiusLow = baseRadius - deltaRadius * ((float) j / stacks);
		radiusHigh = baseRadius - deltaRadius * ((float) (j + 1) / stacks);

		for (i = 0; i <= slices; i++) {
			vertices[x++]=radiusLow * sinCache[i];
			vertices[x++]=radiusLow * cosCache[i];
			vertices[x++]=zLow;
			vertices[x++]=radiusHigh * sinCache[i];
			vertices[x++]=radiusHigh * cosCache[i];
			vertices[x++]=zHigh;
		}
	}
	glVertexPointer(3, GL_FLOAT, 0, vertices);
	glEnableClientState(GL_VERTEX_ARRAY);
	glDrawArrays(GL_TRIANGLE_STRIP, 0, (slices+1) * stacks * 2);
	glDisableClientState(GL_VERTEX_ARRAY);  
}

void Render()
{
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);  
	glLoadIdentity();
	if(drawTriangle){
		glEnable(GL_TEXTURE_2D);
		glColor4x(FixedFromFloat(0.8f),FixedFromFloat(0.8f),FixedFromFloat(0.8f),ONE);
		glVertexPointer(2, GL_FLOAT, 0, bgVertices);
		glTexCoordPointer(2, GL_FLOAT, 0, bgTexCoords);
		glEnableClientState(GL_VERTEX_ARRAY);
		glEnableClientState(GL_TEXTURE_COORD_ARRAY);
		glDrawArrays(GL_TRIANGLE_STRIP, 0, 4);
		glDisableClientState(GL_VERTEX_ARRAY);
		glDisableClientState(GL_TEXTURE_COORD_ARRAY);
		glDisable(GL_TEXTURE_2D);
	}
	drawBoard();
	eglSwapBuffers(glesDisplay, glesSurface);
}



void Perspective (GLfloat fovy, GLfloat aspect, GLfloat zNear,  GLfloat zFar)
{
	GLfixed xmin, xmax, ymin, ymax, aspectFixed, znearFixed;     

	aspectFixed = FixedFromFloat(aspect);
	znearFixed = FixedFromFloat(zNear);

	/*Because we do not have a good fixed point math library, we have to 
	use the standard one, that works with floats, or even worse, 
	doubles. tan function receives floats/doubles and returs 
	float/doubles. Once we get the float, we transform it imediately 
	to fixed point*/
	ymax = MultiplyFixed(znearFixed, FixedFromFloat((GLfloat)tan(fovy * 3.1415962f / 360.0f)));  
	ymin = -ymax;

	xmin = MultiplyFixed(ymin, aspectFixed);
	xmax = MultiplyFixed(ymax, aspectFixed);  
	glFrustumx(xmin, xmax, ymin, ymax, znearFixed, FixedFromFloat(zFar));
}




void SetOrtho()
{
	/*Setup of the orthographic matrix. We use an ortho cube centered 
	at (0,0,0) with 40 units of edge*/
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	/*	
	glOrthox(FixedFromInt(-20), FixedFromInt(20), 
	FixedFromInt(-20), FixedFromInt(20), 
	FixedFromInt(-20), FixedFromInt(20));
	*/
	glOrthox(FixedFromInt(-50), FixedFromInt(50), 
		FixedFromInt(-50), FixedFromInt(50), 
		FixedFromInt(-50), FixedFromInt(50));

	glMatrixMode(GL_MODELVIEW);
	SetWindowText(hWnd, L"OpenGLES ortho");
}


void SetPerspective()
{
	RECT r;
	GetWindowRect(hWnd, &r);    
	float ratio = (float)(r.right - r.left)/(r.bottom - r.top);

	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();      
	Perspective(45.0f, ratio, -10.0f, 50.0f);
	glMatrixMode(GL_MODELVIEW);
	SetWindowText(hWnd, L"OpenGLES perspective");
}

