#ifndef RENDER_H
#define RENDER_H

#include <windows.h> 
#include <GLES/gl.h>
#include <GLES/egl.h>
#include "resource.h"
#include "board.h"

#undef	PI
#define PI	      3.14159265358979323846f
/*changed defines to const because we do not want these operations to occur in run time*/
const unsigned int PRECISION = 16;
const GLfixed ONE  = 1 << PRECISION;
const GLfixed ZERO = 0;

/**
*Converts an int value to a fixed value
*@param value The value to convert
*/
inline GLfixed FixedFromInt(int value) 
{
	return value << PRECISION;
};

/**
*Converts a float value to a fixed value
*@param value The value to convert
*/
inline GLfixed FixedFromFloat(float value) 
{ 
	return static_cast<GLfixed> (value * static_cast<float>(ONE));
};

/**
*Multiplies two fixed values
*@param op1 The first value to multiply
*@param op2 The second value to multiply
*/
inline GLfixed MultiplyFixed(GLfixed op1, GLfixed op2)
{ 
	return (op1 * op2) >> PRECISION;
};

/**
* Initializes the graphics with a default board, used for testing
*/
bool InitOGLES();
/**
* Initializes the graphics with a pointer to a game board
* @param a A pointer to a game board
*/
bool InitDraw(board* a);
/**
* Renders the current game board
*/
void Render();
/**
* Changes the view to orthographic
*/
void SetOrtho();
/**
* Changes the view to perspective
*/
void SetPerspective(); // function to change to perspective mode

/**
*A gluPerspective-like function but modified to work with Glfixed
*/
void Perspective (GLfloat fovy, GLfloat aspect, GLfloat zNear,  GLfloat zFar); 
/**
*Initializes the block mesh and then draws a single block. If the mesh is already initialized
*then it simply draws a block
*@param inner The radius of the inner cylinder
*@param outer The radius of the outer cylinder
*@param height The desired height of the block
*@param slices The number of slices in the cylinder
*/
void initBlock(GLfloat inner, GLfloat outer,GLfloat height, GLint slices);
/**
*Draws the inner and outer cylinders of the game board with the desired number of slices and stacks
*@param slices The number of slices in the cylinder, usually the x dimension of the game board
*@param stacks The numbes of stacks in the cylinder, usually the y dimension of the game board
*/
void drawCylinders(GLint slices, GLint stacks);
/**
*Draws the disk at the top of the game cylinder
*@param innerRadius The radius of the inner cylinder
*@param outerRadius The radius of the outer cylinder
*@param slices The number of slices in the cylinders
*/
void cap(GLfloat innerRadius, GLfloat outerRadius, GLint slices);
/**
*Draws a cylinder with the given dimensions. A difference between the top and bottom radius creates
*a cone
*@param baseRadius The radius at the bottom of the cylinder
*@param topRadius The radius at the top of the cylinder
*@param height The height of the cylinder
*@param slices The number of slices in the cylinder, the more slices, the more cylindrical the approximation
*@param stacks The number of stacks in the cylinder
*/
void Cylinder(GLfloat baseRadius, GLfloat topRadius, GLfloat height, GLint slices, GLint stacks);
#endif