/**
 * Module		: board.cpp
 * Author		: Christopher M. Donnelly
 * Email		: cmdonn@wpi.edu
 * Course		: Embedded Game Design MQP
 *
 * Description	: acts as a game board
 *
 * Date			: 3/14/2008
 *
 * History:
 * Revision		Date			Changed By
 * --------		-----------		----------
 * 00.01		3/14/2008		cmdonn
 * 00.02		3/21/2008		cmdonn
 *
 *
 *
 *
 **/

#include "board.h"


	/**
	 * a board keeps track of the location of all of the spheres in
	board::board()
	{

		board(SIZE_X,SIZE_Y); //replace later with config file code
	}

	/**
	 *constructor creates a board of size x by y all declared to a value of NOTHING.
	 * @param x The number of columns the board will have
	 * @param y The number of rows the board will have
	 */
	board::board(int x, int y)
	{

		x_axis = x;
		y_axis = y;
		score = 0;
		activePiece = cobli();

		game = (sphere***) calloc(x_axis, sizeof(sphere**));

		for (char i = 0; i < x_axis; i++)
		{

			game[i] = (sphere**)calloc(y_axis, sizeof(sphere*));

			for(char j = 0; j < y_axis; j++)
			{
				game[i][j] = NOTHING;
			}
		}
		for (char i = 0; i < x_axis; i++)
		{
			for(char j = 0; j < y_axis; j++)
			{
				printf("%c", game[i][j]->color);
			}
			printf("\n");
		}



	}


	/**
	 * returns the array which shows the state of the board, including the cobli.
	 *
	 * @return the board state
	 */
	sphere*** board::getboard()
	{
		return game;
	}

	void board::printboard()
	{
		for(int i = 0; i < 5; i++)
		{
			for(int j =0; j < 5; j++)
			{
				printf("%c",activePiece.piece[i][j]->color);
			}
			printf("\n");
		}
	}
	/**
	 * Drops the cobli down one slot and returns whether the drop was successful
	 * or if it hit the bottom of the board or a playing piece
	 * @returns false when a drop hits a piece and another piece is off the board.  Otherwise, it returns true
	 *
	 */
	bool board::drop()
	{
		removeCobli();
		activePiece.location_x++;
		if(overlap())
		{
			//printf("OVERLAPS!!!\n");
			activePiece.location_x--;

			//check for lose here later
			if(activePiece.location_x< 5)
			{
				for(int i = 0; i< (5-activePiece.location_x); i++)
				{
					for(int j = 0; j <5; j++)
					{
						if(activePiece.piece[i][j]->color != NONE_)
						{
							return false;
						}
					}
				}
			}
			setCobli();
			scoreBoard();
			newPiece();

		}
		setCobli();
		return true;
	}

	/**
	 * Returns whether or not a sphere is overlapping with the cobli.  Note: make
	 * sure that the cobli has not been set with set cobli before running this
	 * or else it will return true
	 * @return true a sphere in the  cobli overlaps with a sphere on the board
	 *		   false: otherwise
	 */
	bool board::overlap()
	{
		//printf("Entered overlap function\n");
		/*if(activePiece.location_x >=x_axis)
		{
			return true;
		}*/
		if (activePiece.location_x < 4)
		{
			for(char i = 4-activePiece.location_x; i < 5; i++)
			{
				for(char j = 0; j < 5; j++)
				{
					if(activePiece.piece[i][j]->color != NONE_)
					{
						if(game[activePiece.location_x - (4-i)][(j+activePiece.location_y)%y_axis]->color != NONE_)
						{
							return true;
						}
					}
				}
			}
		}
		else
		{
			for(char i = 0; i< 5; i++)
			{
				for(char j = 0; j < 5; j++)
				{
					if(activePiece.piece[i][j]->color != NONE_)
					{
						if(activePiece.location_x-x_axis >= 4-i)//returns if crossed end of board
						{
							return true;
						}
						if( game[activePiece.location_x - (4-i)][(j+activePiece.location_y)%y_axis]->color != NONE_)
						{
							return true;
						}

					}
				}
			}
		}
		return false;
	}

	void board::removeCobli()
	{
		/*
		 *4 is the minimal part where the entire cobli would be visible
		 */
		if (activePiece.location_x < 4)
		{
			for(char i = 4-activePiece.location_x; i < 5; i++)
			{
				for(char j = 0; j < 5; j++)
				{
					if(activePiece.piece[i][j]->color != NONE_)
					{
						game[activePiece.location_x - (4-i)][(j+activePiece.location_y)%y_axis] = NOTHING;
					}
				}
			}
		}
		else
		{
			for(char i = 0; i< 5; i++)
			{
				for(char j = 0; j < 5; j++)
				{
					if(activePiece.piece[i][j]->color != NONE_)
					{
						game[activePiece.location_x - (4-i)][(j+activePiece.location_y)%y_axis] = NOTHING;
					}
				}
			}
		}

	}

	void board::setCobli()
	{
		/*
		 *4 is the minimal part where the entire cobli would be visible
		 */
		if (activePiece.location_x < 4)
		{
			for(char i = 4-activePiece.location_x; i < 5; i++)
			{
				for(char j = 0; j < 5; j++)
				{
					if(activePiece.piece[i][j]->color != NONE_)
					{
						game[activePiece.location_x - (4-i)][(j+activePiece.location_y)%y_axis] = activePiece.piece[i][j];
					}
				}
			}
		}
		else
		{
			for(char i = 0; i< 5; i++)
			{
				for(char j = 0; j < 5; j++)
				{
					if(activePiece.piece[i][j]->color != NONE_)
					{
						game[activePiece.location_x - (4-i)][(j+activePiece.location_y)%y_axis] = activePiece.piece[i][j];
					}
				}
			}
		}
	}

	void board::rotateLeft()
	{
		removeCobli();
		activePiece.rotateLeft();
		if(overlap())
		{
			activePiece.rotateRight();
		}
		setCobli();
	}
	void board::rotateRight()
	{
		removeCobli();
		activePiece.rotateRight();
		if(overlap())
		{
			activePiece.rotateLeft();
		}
		setCobli();
	}

	void board::shiftLeft()
	{
		removeCobli();
		activePiece.location_y--;
		if(activePiece.location_y < 0)
		{
			activePiece.location_y = y_axis -1;
		}
		if (overlap())
		{
			activePiece.location_y++;
		}
		setCobli();
	}

	void board::shiftRight()
	{
		removeCobli();
		activePiece.location_y++;
		if (overlap())
		{
			activePiece.location_y--;
		}
		setCobli();
	}
	void board::newPiece()
	{
		char location = activePiece.location_y;
		activePiece.generateNewCobli();
		activePiece.location_y = location;
	}

	int board::get_x()
	{
		return x_axis;
	}

	int board::get_y()
	{
		return y_axis;
	}

	int board::getCobliPosition()
	{
		return activePiece.location_y;
	}

	int board::getScore()
	{
		return score;
	}

	void board::scoreBoard()
	{
		int tempScore = 0;
		bool destroyRow = false;
		for(char i = 0; i < x_axis; i++)
		{
			for(char j = 0; j < y_axis; j++)
			{
				if(destroyRow)
				{
					tempScore+= game[i][j]->score;

					for(char k = i; k > 0; k--)
					{
						game[k][j] = game [k-1][j];
					}
					game[0][j] = NOTHING;
				}
				else
				{
					if( game[i][j]->color != NONE_)
					{
						if(j+1 == y_axis)
						{
							destroyRow = true;
							j = -1;
						}
					}
					else
					{
						tempScore = 0;
						break;
					}
				}
			}
			destroyRow = false;
		}
		score += tempScore;
	}

	/**
	 *destructor.  Hopefully will keep memory leaks down.
	 */
	board::~board()
	{
		for (char i = 0; i < x_axis; i++)
		{
			for(char j = 0; j < y_axis; j++)
			{
				//delete (game[i][j]);//(got rid of new, so delete is not needed...i think
			}
			free (game[i]);
			game[i] = NULL;
		}
		free(game);
		game = NULL;
	}
