
#include "camera.h"
#include "render.h"
#include <math.h>
#define M_PI 3.14159265358

camera::camera()
{
}

void camera::setModelViewMatrix()
{
  GLfixed m[16];
  Vector3 eVec(eye.x, eye.y, eye.z);
  m[0] = u.x; m[4] = u.y; m[8] = u.z; m[12] = -eVec.dot(u);
  m[1] = v.x; m[5] = v.y; m[9] = v.z; m[13] = -eVec.dot(v);
  m[2] = n.x; m[6] = n.y; m[10] = n.z;m[14] = -eVec.dot(n);
  m[3] = 0;   m[7] = 0;   m[11] = 0;  m[15] = 1.0;

  glMatrixMode(GL_MODELVIEW);
  glLoadMatrixx(m);

}

void camera::set(Point3 Eye, Point3 look, Vector3 up)
{
  eye.set(Eye);
  n.set(eye.x - look.x, eye.y - look.y, eye.z - look.z);
  u.set(up.cross(n));
  n.normalize(); u.normalize();  //make them unit length
  v.set(n.cross(u));
  setModelViewMatrix();
}


void camera::slide(GLfixed delU, GLfixed delV, GLfixed delN)
{
  eye.x += delU * u.x + delV * v.x + delN * n.x;
  eye.y += delU * u.y + delV * v.y + delN * n.y;
  eye.z += delU * u.z + delV * v.z + delN * n.z;
  
  look.z += delU * u.z + delV * v.z + delN * n.z;
  look.z += delU * u.z + delV * v.z + delN * n.z;
  look.z += delU * u.z + delV * v.z + delN * n.z;
  setModelViewMatrix();
}


void camera::roll(GLfixed angle)
{
  GLfixed cs = FixedFromFloat(cos(M_PI / 180 * angle));

  GLfixed sn = FixedFromFloat(sin(M_PI / 180 * angle));
  Vector3 t(u);
  u.set(cs * t.x - sn * v.x, cs * t.y - sn * v.y, cs * t.z - sn * v.z);
  v.set(sn * t.x + cs * v.x, sn * t.y + cs * v.y, sn * t.z + cs * v.z);

  setModelViewMatrix();

}

void camera::setShape(GLfixed vAng, GLfixed asp, GLfixed nearD, GLfixed farD)
{
  viewAngle = vAng;
  aspect = asp;
  nearDist = nearD;
  farDist = farD;
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();

  Perspective(viewAngle, aspect, nearDist, farDist);
  glMatrixMode(GL_MODELVIEW);
}

void camera::yaw(GLfixed angle)
{
  GLfixed cs = FixedFromFloat(cos(M_PI / 180 * angle));

  GLfixed sn = sin(M_PI / 180 * angle);
  Vector3 t(n);
  n.set(cs * t.x - sn * u.x, cs * t.y - sn * u.y, cs * t.z - sn * u.z);
  u.set(sn * t.x + cs * u.x, sn * t.y + cs * u.y, sn * t.z + cs * u.z);

  setModelViewMatrix();
}

void camera::pitch(GLfixed angle)
{
  GLfixed cs = cos(M_PI / 180 * angle);

  GLfixed sn = sin(M_PI / 180 * angle);
  Vector3 t(n);
  n.set(cs * t.x - sn * v.x, cs * t.y - sn * v.y, cs * t.z - sn * v.z);
  v.set(sn * t.x + cs * v.x, sn * t.y + cs * v.y, sn * t.z + cs * v.z);

  setModelViewMatrix();
}
