 /**
  * A class, which when created controls all rotations and movement of cobli,
  * as well as it's appearances contains a 5x5 matrix of spheres
  * @author cmdonn@wpi.edu
  * @version 1.6
  *
  **/
 #include "cobli.h"

/**
 * Creates a cobli for use with the board object.  Instanciates a cobli filled
 * with NOTHING
 *
 */
cobli::cobli()
{
	location_x = 0;
	location_y = 2;
	for(int x = 0; x< 5; x++)
	{
		for(int y = 0; y< 5; y++)
		{
			piece[x][y] = NOTHING;
		}
	}
}

/**
 * Returns the location of the center of the cobli such that if this is
 * the center,
 * @return a char showing the location of the center of the cobli
 */
char cobli::get_location()
{
	return location_y + 2;
}

/**
 * adds the pointer to sphere a to location x,y of the cobli
 *
 * @param a a pointer to a sphere which must exist.  Will be stored in cobli
 * @param x the column a is going.  May throw an error if not between 0 and 4
 * @param y the row a is going.  May throw an error if not between 0 and 4
 */
void cobli::addSphere(sphere* a, int x, int y)
{
	if(0<=x && x<5)
	{
		if(0<=y && y < 5)
		{
			piece[x][y] = a;
		}
	}
}

/**
 * generates a new cobli with between 2 and 5 colored spheres of random colors
 * and shape in it
 */
void cobli::generateNewCobli()
{
	location_x = 0;
	location_y = 0;
	char x = 2;
	char y = 2;
	char random_char;
	for(int x = 0; x< 5; x++)
	{
		for(int y = 0; y< 5; y++)
		{
			piece[x][y] = NOTHING;
		}
	}
	bool change = true;

	//put's up to 5 sphere's into a cobli
	for(char i = 0; i<5; i++)
	{
		//move in a random direction or hit the edge
		if (change)
		{
			random_char = rand();
			if(random_char < 0)
			{
				random_char = 0-random_char;
			}
			random_char %= 5;
			if(random_char == 0)
			{
				piece[x][y] = RED_SPHERE;
			}
			else if(random_char == 1)
			{
				piece[x][y] = GREEN_SPHERE;
			}
			else if(random_char == 2)
			{
				piece[x][y] = BLUE_SPHERE;
			}
			else if(random_char == 3)
			{
				piece[x][y] = YELLOW_SPHERE;
			}
			else if(random_char == 4)
			{
				piece[x][y] = ORANGE_SPHERE;
			}
		}
		random_char = rand();
		if(random_char < 0)
		{
			random_char = 0 - random_char;
		}
		random_char %= 4;
		if(random_char == 0)
		{
			if(x != 0)
			{
				x--;
				change = true;
			}
		}
		else
		{
			if(random_char == 1)
			{
				if(x != 4)
				{
					x++;
					change = true;
				}
			}
			else
			{
				if(random_char == 2)
				{
					if(y != 0)
					{
						y--;
						change = true;
					}
				}
				else
				{
					if(random_char == 3)
					{
						if(y != 4)
						{
							y++;
							change = true;
						}
					}
					else
					{
						change = false;
					}
				}
			}
		}
	}



}


/**
 * Rotates the spheres in the cobli 90 degrees clockwise.
 */
void cobli::rotateRight()
{
	sphere* newpositions[5][5];
	for(int x = 0; x<5; x++)
	{
		for(int y = 0; y < 5; y++)
		{
			 newpositions[y][4-x] = piece[x][y];
		}
	}
	//this for loop may cause odd memory things.  If it does, we should fix it.
	for(int x = 0; x<5; x++)
	{
		for(int y = 0; y < 5; y++)
		{
			piece[x][y] = newpositions[x][y];
		}
	}
	//here we should destroy the array if we have to.

}

/**
 * Rotates the spheres in the cobli 90 degrees counter-clockwise
 */
void cobli::rotateLeft ()
{
	sphere* newpositions[5][5];
	for(int x = 0; x<5; x++)
	{
		for(int y = 0; y < 5; y++)
		{
			 newpositions[4-y][x] = piece[x][y];
		}
	}

	//same notes as above and below
	for(int x = 0; x<5; x++)
	{
		for(int y = 0; y < 5; y++)
		{
			piece[x][y] = newpositions[x][y];
		}
	}
}

/**
 * rotates the spheres in the cobli 180 degrees
 */
void cobli::rotate180 ()
{
	sphere* newpositions[5][5];
	for(int x = 0; x<5; x++)
	{
		for(int y = 0; y < 5; y++)
		{
			 newpositions[4-x][4-y] = piece[x][y];
		}
	}
	//this for loop may cause odd memory things.  If it does, we should fix it.
	for(int x = 0; x<5; x++)
	{
		for(int y = 0; y < 5; y++)
		{
			piece[x][y] = newpositions[x][y];
		}
	}
	//here we should destroy the array if we have to.
}

