#include "main.h"
#include "render.h"
#include <commctrl.h>
#include <time.h>
HINSTANCE hInst;
HWND hWnd;
HDC hDC; 
board* GameBoard;
//************************************
//Contains the messages to be handled
//************************************
const struct decodeUINT MainMessages[] =
{
	WM_CREATE, DoCreateMain,
//	WM_PAINT, DoCreateMain,
	WM_LBUTTONDOWN, DoLeftButton,
	WM_KEYDOWN, DoKeyDown,
	WM_COMMAND, DoMenuCommand
};

TCHAR szAppName[] = TEXT("Tubetris");

/* This variable will change with every right mouse click in the touch screen of the pda */
bool drawTriangle = false;


//**************
//Main function
//**************
int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPTSTR lpCmdLine,	int nCmdShow)
{
	MSG msg;
	WNDCLASS	wc;
	hInst = hInstance;
	bool done = false; 
	if(hWnd = FindWindow(szAppName, NULL)) 
	{
		SetForegroundWindow((HWND)((ULONG) hWnd | 0x00000001));
		return -1;
	}
	
	
	wc.style          = CS_HREDRAW | CS_VREDRAW; 
	wc.lpfnWndProc    = (WNDPROC) WndProc;
	wc.cbClsExtra     = 0;
	wc.cbWndExtra     = 0;
	wc.hInstance      = hInstance;
	wc.hIcon          = LoadIcon(hInstance, NULL);
	wc.hCursor	  = 0; 
	wc.hbrBackground  = 0;
	wc.lpszMenuName	  = NULL;
	wc.lpszClassName  = szAppName;
	
	if(!RegisterClass(&wc))
	{
		return false;
	}
	
	hWnd = CreateWindow(szAppName,
		szAppName,                    
		WS_VISIBLE,
		CW_USEDEFAULT, CW_USEDEFAULT,
		CW_USEDEFAULT, CW_USEDEFAULT, 
		NULL, NULL, hInst, NULL); 
	
	if(!hWnd) return false;
	GameBoard = new board(8,8);
	GameBoard->newPiece();
	if(!InitDraw(GameBoard)) return false;  
	int x = 0;
	SetWindowText(hWnd, szAppName);
	ShowWindow(hWnd, nCmdShow); 
	UpdateWindow(hWnd);
	while(!done)
	{
		if(PeekMessage(&msg, NULL, 0, 0, PM_REMOVE))
		{
			if(msg.message == WM_QUIT)
			{
				done = true;
			}
			else
			{ 
				TranslateMessage(&msg);
				DispatchMessage(&msg);
			}
		}
		x++;
		if(x > 5){
			x = 0;
		if(!GameBoard->drop())
		{
			
			MessageBox(NULL, L"GAME OVER!!!1", L"OpenGL ES", MB_ICONEXCLAMATION | MB_OK);
			done = true;
		}
		}
		Render();
		
	}
	
	Clean();
	//Brought here the window destruction
	DestroyWindow(hWnd);
	UnregisterClass(szAppName, hInst);  
	return 0;
}




//*************************
//Message handling routine
//*************************
LRESULT CALLBACK WndProc(HWND hWnd, UINT wMsg, WPARAM wParam, LPARAM lParam)
{
    // Search message list to see if we need to handle this
    // message.  If in list, call procedure.

    for (int i = 0; i < dim(MainMessages); i++) 
	{
        if (wMsg == MainMessages[i].Code)
		{
            return (*MainMessages[i].Fxn)(hWnd, wMsg, wParam, lParam);
		}
    }
    return DefWindowProc (hWnd, wMsg, wParam, lParam);
}


//******************************************
//This function is suppose to create a menu
//******************************************
LRESULT DoCreateMain(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{		
	HMENU hMainMenu = CreateMenu ();
	HMENU hMenu = CreatePopupMenu ();
	AppendMenu (hMenu, MF_STRING | MF_ENABLED, IDM_NEWGAME, TEXT ("&New"));
	AppendMenu (hMenu, MF_STRING | MF_ENABLED, 101, TEXT ("&Triangle"));

/*	AppendMenu (hMenu, MF_STRING | MF_ENABLED, 102, TEXT ("&Box"));*/
	AppendMenu (hMenu, MF_STRING | MF_ENABLED, 103, TEXT ("E&xit"));

	AppendMenu (hMainMenu, MF_STRING | MF_ENABLED | MF_POPUP, (UINT)hMenu, TEXT ("&Game"));

	hMenu = CreatePopupMenu ();
	AppendMenu (hMenu, MF_STRING | MF_ENABLED, 107, TEXT ("&About"));

	AppendMenu (hMainMenu, MF_STRING | MF_ENABLED | MF_POPUP, (UINT)hMenu, TEXT ("&Help"));

	HWND hwndCB;
	// Create a command bar.
	hwndCB = CommandBar_Create (hInst, hWnd, IDC_CMDBAR);

	CommandBar_InsertMenubarEx(hwndCB, 0, (LPTSTR)hMainMenu, 1);

	// Add exit button to command bar. 
	CommandBar_AddAdornments (hwndCB, 0, 0);




//	ValidateRect(hWnd, NULL); //Needed to avoid new WM_PAINT messages
	return 0; 
}		


//******************************
//Handles the left mouse button
//******************************
LRESULT DoLeftButton(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
//	MessageBox(NULL, L"OpenGL ES", L"destroy message has been posted!",	MB_ICONEXCLAMATION | MB_OK);
	drawTriangle = !drawTriangle;
	return 0;
}


//*********************************************
//Handles the up, down, left and right buttons
//*********************************************
LRESULT DoKeyDown(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	switch(wParam)
	{
	case VK_UP:
		{
			
			GameBoard->rotateLeft();
			return 0;
		}
	case VK_DOWN:
		{
			GameBoard->drop();
			return 0;
		}

	case VK_LEFT:
		{
			GameBoard->shiftLeft();
			return 0;
		}

	case VK_RIGHT:
		{
			GameBoard->shiftRight();
			return 0;
		}

	case VK_RETURN:
		{
			MessageBox(NULL, TEXT("Paused"), TEXT("OpenGL ES"), MB_ICONEXCLAMATION | MB_OK);
			return 0;
		}
	}
	return 0;
}


//**************************
//Handles the menu commands
//**************************
LRESULT DoMenuCommand(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	//extracting the menu item ID
	int id = LOWORD(wParam);
	switch(id)
	{
	case 101:
		{
			drawTriangle = true;
		}
		return 0;
	case 102:
		{
			drawTriangle = false;
		}
		return 0;
	case 103:
		exit(0);
		return 0;
	}
	return 0;
}
